"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.SdkIndexLinkClientPlugin = void 0;
const tslib_1 = require("tslib");
const fs_1 = require("fs");
const path_1 = require("path");
const typedoc_1 = require("typedoc");
const isClientModel = (model) => { var _a, _b; return (_b = (_a = model === null || model === void 0 ? void 0 : model.sources) === null || _a === void 0 ? void 0 : _a[0]) === null || _b === void 0 ? void 0 : _b.fullFileName.includes(`${path_1.sep}clients${path_1.sep}`); };
class SdkIndexLinkClientPlugin {
    constructor(options, logger, renderer) {
        this.options = options;
        this.logger = logger;
        this.renderer = renderer;
        this.onConverterEnd = async (context) => {
            const project = context.project;
            if (!project.groups) {
                project.groups = [];
            }
            let group = project.groups.find((value) => value.title === this.defaultGroup);
            if (!group) {
                group = new typedoc_1.ReflectionGroup(this.defaultGroup);
                project.groups.push(group);
            }
            const modules = project.getChildrenByKind(typedoc_1.ReflectionKind.SomeModule);
            await this.registerClients(project, modules);
            group.categories = this.defineCategories(group, modules);
        };
        this.onPageBegin = (page) => {
            if (page.model.flags.includes("SkipRendering")) {
                page.preventDefault();
            }
            if (page.model.isProject()) {
                const group = page.model.groups.find((value) => value.title === this.defaultGroup);
                const clientsCategory = group.categories.find((value) => value.title === "Clients");
                for (const child of clientsCategory.children || []) {
                    child.url = `https://docs.aws.amazon.com/AWSJavaScriptSDK/v3/latest/clients/${child.originalName}/`;
                    child.name = `@aws-sdk/${child.originalName.replace("client-", "")}`;
                }
            }
        };
        this.renderer.application.converter.on(typedoc_1.Converter.EVENT_END, this.onConverterEnd);
        this.renderer.on(typedoc_1.Renderer.EVENT_BEGIN_PAGE, this.onPageBegin);
    }
    async registerClients(project, modules) {
        return new Promise((resolvePromise, rejectPromise) => {
            const clientPath = (0, path_1.resolve)("./clientDocs/clients");
            (0, fs_1.readdir)(clientPath, (err, clientFiles) => {
                if (err)
                    rejectPromise(err);
                clientFiles.forEach((clientName) => {
                    if (clientName.includes("client") && !clientName.includes("documentation")) {
                        const clientDeclaration = new typedoc_1.DeclarationReflection(clientName, typedoc_1.ReflectionKind.Module, project);
                        project.registerReflection(clientDeclaration);
                        modules.push(clientDeclaration);
                    }
                });
                resolvePromise();
            });
        });
    }
    defineCategories(group, reflections) {
        const categories = group.categories || [];
        if (this.isCategorized(categories))
            return group.categories;
        const clients = new typedoc_1.ReflectionCategory("Clients");
        const packages = new typedoc_1.ReflectionCategory("Packages");
        const libs = new typedoc_1.ReflectionCategory("Libraries");
        reflections.forEach((reflection) => {
            if (this.isClient(reflection)) {
                clients.children.push(reflection);
                reflection.flags.includes("SkipRendering");
            }
            else if (this.isLib(reflection)) {
                libs.children.push(reflection);
            }
            else {
                packages.children.push(reflection);
            }
        });
        categories.push(...[clients, libs, packages]);
        return categories;
    }
    isCategorized(categories) {
        const childrenNames = categories.map((child) => child.title);
        return (childrenNames.includes("Clients") && childrenNames.includes("Packages") && childrenNames.includes("Libraries"));
    }
    isClient(item) {
        return isClientModel(item) || item.name.includes("client-");
    }
    isLib(item) {
        var _a;
        return (_a = item === null || item === void 0 ? void 0 : item.sources) === null || _a === void 0 ? void 0 : _a[0].fileName.startsWith(`lib${path_1.sep}`);
    }
}
tslib_1.__decorate([
    (0, typedoc_1.BindOption)("out")
], SdkIndexLinkClientPlugin.prototype, "outputDirectory", void 0);
tslib_1.__decorate([
    (0, typedoc_1.BindOption)("clientDocs")
], SdkIndexLinkClientPlugin.prototype, "clientDocs", void 0);
tslib_1.__decorate([
    (0, typedoc_1.BindOption)("defaultGroup")
], SdkIndexLinkClientPlugin.prototype, "defaultGroup", void 0);
exports.SdkIndexLinkClientPlugin = SdkIndexLinkClientPlugin;
